﻿using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Forms;
using AddinExpress.MSO;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace OutlookItemEventsDemo
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("C6CFD2E1-398E-4997-8D85-D4A69E2F1995"), ProgId("OutlookItemEventsDemo.AddinModule")]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        private OutlookItemEventsClass outlookItemEvents = null;
        private bool canConnect = true;

        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
        }

        private ADXOutlookAppEvents adxOutlookEvents;

        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;

        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.adxOutlookEvents = new AddinExpress.MSO.ADXOutlookAppEvents(this.components);
            // 
            // adxOutlookEvents
            // 
            this.adxOutlookEvents.ItemSend += new AddinExpress.MSO.ADXOlItemSend_EventHandler(this.adxOutlookEvents_ItemSend);
            this.adxOutlookEvents.InspectorActivate += new AddinExpress.MSO.ADXOlInspector_EventHandler(this.adxOutlookEvents_InspectorActivate);
            this.adxOutlookEvents.InspectorClose += new AddinExpress.MSO.ADXOlInspector_EventHandler(this.adxOutlookEvents_InspectorClose);
            this.adxOutlookEvents.ExplorerActivate += new AddinExpress.MSO.ADXOlExplorer_EventHandler(this.adxOutlookEvents_ExplorerActivate);
            this.adxOutlookEvents.ExplorerClose += new AddinExpress.MSO.ADXOlExplorer_EventHandler(this.adxOutlookEvents_ExplorerClose);
            this.adxOutlookEvents.ExplorerSelectionChange += new AddinExpress.MSO.ADXOlExplorer_EventHandler(this.adxOutlookEvents_ExplorerSelectionChange);
            // 
            // AddinModule
            // 
            this.AddinName = "OutlookItemEventsDemo";
            this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
            this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);
            this.AddinBeginShutdown += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinBeginShutdown);

        }
        #endregion

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }

        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }

        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }

        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

        private void AddinModule_AddinStartupComplete(object sender, EventArgs e)
        {
            outlookItemEvents = new OutlookItemEventsClass(this);
        }

        private void adxOutlookEvents_InspectorActivate(object sender, object inspector, string folderName)
        {
            if (canConnect)
            {
                Outlook._Inspector olInsp = (Outlook._Inspector)inspector;
                object item = olInsp.CurrentItem;

                if (item is Outlook._MailItem)
                {
                    outlookItemEvents.ConnectTo(item, true);
                }
            }
        }

        private void ConnectToSelectedItem(object explorer)
        {
            Outlook.Selection sel = null;

            try
            {
                Outlook._Explorer expl = explorer as Outlook._Explorer;
                sel = expl.Selection;
                if (sel.Count == 1)
                {
                    object item = sel[1];
                    if (item is Outlook._MailItem)
                        outlookItemEvents.ConnectTo(item, true);
                    else
                        Marshal.ReleaseComObject(item);
                }
            }
            finally
            {
                if (sel != null)
                    Marshal.ReleaseComObject(sel);
            }
        }

        private void adxOutlookEvents_ExplorerActivate(object sender, object explorer)
        {
            ConnectToSelectedItem(explorer);
        }

        private void adxOutlookEvents_ExplorerSelectionChange(object sender, object explorer)
        {
            ConnectToSelectedItem(explorer);
        }

        private void adxOutlookEvents_ExplorerClose(object sender, object explorer)
        {
            int count = 0;
            Outlook._Explorers expls = null;

            try
            {
                expls = OutlookApp.Explorers;
                count = expls.Count;
            }
            catch
            {
            }
            finally
            {
                if (expls != null)
                    Marshal.ReleaseComObject(expls);
            }

            if (count == 0)
                canConnect = false;
        }

        private void adxOutlookEvents_ItemSend(object sender, ADXOlItemSendEventArgs e)
        {
            if (outlookItemEvents != null)
                outlookItemEvents.RemoveConnection();
        }

        private void adxOutlookEvents_InspectorClose(object sender, object inspector, string folderName)
        {
            if (outlookItemEvents != null)
                outlookItemEvents.RemoveConnection();
        }

        private void AddinModule_AddinBeginShutdown(object sender, EventArgs e)
        {
            if (outlookItemEvents != null)
                outlookItemEvents.RemoveConnection();
        }

    }
}

