﻿using System;
using System.Windows.Forms;
using AddinExpress.MSO;
using Outlook = Microsoft.Office.Interop.Outlook;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace OutlookItemEventsDemo
{
    /// <summary>
    /// Add-in Express Outlook Item Events Class
    /// </summary>
    public class OutlookItemEventsClass : AddinExpress.MSO.ADXOutlookItemEvents
    {
        public OutlookItemEventsClass(AddinExpress.MSO.ADXAddinModule module)
            : base(module)
        {
        }


        public override void ProcessBeforeAttachmentAdd(object attachment,
            AddinExpress.MSO.ADXCancelEventArgs e)
        {
            Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
            int sizeInBytes = myAttachment.Size;
            int sizeInKiloBytes = sizeInBytes / 1024;

            //Check if attachment is larger than 1MB
            if (sizeInKiloBytes >= 1024)
            {
                MessageBox.Show("PLEASE NOTE: Attachment must be under 1MB. Attachment will be removed.",
                    "Attachment size warning");
                e.Cancel = true;
            }
        }

        public override void ProcessAttachmentAdd(object attachment)
        {
            Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
            int sizeInBytes = myAttachment.Size;
            int sizeInKiloBytes = sizeInBytes / 1024;

            //Check if attachment is larger than 1MB
            if (sizeInKiloBytes >= 1024)
            {
                MessageBox.Show("PLEASE NOTE: To keep mailbox sizes low, try and keep attachment under 1MB",
                    "Attachment size warning");
            }
        }

        public override void ProcessAttachmentRead(object attachment)
        {
            Outlook._JournalItem journalItem = null;

            try
            {
                Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
                string fileName = myAttachment.FileName;

                journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                journalItem.Subject = "You opened " + fileName + " today";
                journalItem.Save();
            }
            finally
            {
                if (journalItem != null)
                    Marshal.ReleaseComObject(journalItem);
            }
        }

        public override void ProcessAttachmentRemove(object attachment)
        {
            Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
            Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;

            string fileName = String.Empty;
            Outlook._JournalItem journalItem = null;

            try
            {
                fileName = myAttachment.FileName;

            }
            catch (Exception)
            {
            }

            if (!String.IsNullOrEmpty(fileName))
            {
                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "You removed " + fileName + " today";
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null)
                        Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        public override void ProcessBeforeAttachmentPreview(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
            Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
            string fullPath = @"C:\Temp\" + myAttachment.FileName;
            myAttachment.SaveAsFile(fullPath);
        }

        public override void ProcessBeforeAttachmentRead(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
            Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
            string fileName = myAttachment.FileName;

            if (fileName.ToLower().Contains("virus"))
            {
                MessageBox.Show("WARNING: The file appears dangerous. Better not open it.",
                    "Warning - Dangerous file");
                e.Cancel = true;
            }
        }

        public override void ProcessBeforeAttachmentWriteToTempFile(object attachment,
            AddinExpress.MSO.ADXCancelEventArgs e)
        {
            Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
            Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
            string fileName = myAttachment.FileName;

            if (fileName.ToLower().Contains("virus"))
            {
                MessageBox.Show("WARNING: The file appears dangerous. Better not open it.",
                    "Warning - Dangerous file");
                e.Cancel = true;
            }
        }

        public override void ProcessBeforeAttachmentSave(object attachment, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            Outlook.Attachment myAttachment = (Outlook.Attachment)attachment;
            Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;
            string fileName = myAttachment.FileName;

            if (MessageBox.Show("You've made changes to the attached file. Would you like to save it to the central file system?",
                "File Changed", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                myAttachment.SaveAsFile(@"K:\CentralStorage\" + fileName);
            }
        }

        public override void ProcessBeforeCheckNames(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook.Recipient recipient = null;
                Outlook.Recipients recipients = null;

                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;

                try
                {
                    recipients = mail.Recipients;
                    for (int i = 1; i <= recipients.Count; i++)
                    {
                        recipient = null;

                        try
                        {
                            recipient = recipients[i];
                        }
                        finally
                        {
                            if (recipient != null)
                                Marshal.ReleaseComObject(recipient);
                        }
                    }
                }
                finally
                {
                    if (recipients != null)
                        Marshal.ReleaseComObject(recipients);
                }
            }
        }

        public override void ProcessPropertyChange(string name)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)this.ItemObj;

                Outlook.ItemProperty property = null;
                Outlook.ItemProperties properties = null;

                try
                {
                    properties = mail.ItemProperties;
                    property = properties[name];

                    MessageBox.Show(String.Format("You've changed {0} to {1}",
                        name, property.Value));
                }
                finally
                {
                    if (properties != null)
                        Marshal.ReleaseComObject(properties);
                    if (property != null)
                        Marshal.ReleaseComObject(property);
                }
            }
        }

        public override void ProcessCustomPropertyChange(string name)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)this.ItemObj;

                Outlook.UserProperty userProperty = null;
                Outlook.UserProperties properties = null;

                try
                {
                    properties = mail.UserProperties;
                    userProperty = properties.Find(name);
                    if (userProperty != null)
                    {
                        MessageBox.Show(String.Format("You've changed the user property {0} to {1}",
                        name, userProperty.Value));
                    }
                }
                finally
                {
                    if (userProperty != null)
                        Marshal.ReleaseComObject(userProperty);
                    if (properties != null)
                        Marshal.ReleaseComObject(properties);
                }
            }
        }

        public override void ProcessCustomAction(object action, object response, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessReply(object response,
            AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (response is Outlook._MailItem)
            {
                Outlook._MailItem mailResponse = (Outlook._MailItem)response;
                mailResponse.BCC = "someone@somewhere.com";
            }
        }

        public override void ProcessReplyAll(object response,
            AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (response is Outlook._MailItem)
            {
                Outlook._MailItem mailResponse = (Outlook._MailItem)response;
                mailResponse.BCC = "everyone@everywhere.com";
            }
        }

        public override void ProcessForward(object forward,
            AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (forward is Outlook._MailItem)
            {
                Outlook._MailItem mailForward = (Outlook._MailItem)forward;
                if (!mailForward.Subject.ToLower().Contains("confidential"))
                {
                    mailForward.BCC = "mydropbox@work.com";
                }
                else
                {
                    e.Cancel = true;
                }
            }
        }

        public override void ProcessOpen(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mailOpen = (Outlook._MailItem)ItemObj;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;

                Outlook._JournalItem journalItem = null;

                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "You've read a mail with Subject: " + mailOpen.Subject + " today";
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null)
                        Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        public override void ProcessClose(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;
                if (mail.SenderEmailAddress != null &&
                    mail.SenderEmailAddress.Equals("TheBoss@work.com"))
                {
                    e.Cancel = true;
                }
            }
        }

        public override void ProcessRead()
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;

                Outlook._JournalItem journalItem = null;

                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "You've read a mail with Subject: " + mail.Subject + " today";
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null)
                        Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        public override void ProcessWrite(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;

                Outlook._JournalItem journalItem = null;

                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "Mail Saved: " + mail.Subject;
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null)
                        Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        public override void ProcessSend(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;
                Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;

                Outlook._JournalItem journalItem = null;

                try
                {
                    journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                    journalItem.Subject = "Sent an e-mail to " + mail.To;
                    journalItem.Save();
                }
                finally
                {
                    if (journalItem != null)
                        Marshal.ReleaseComObject(journalItem);
                }
            }
        }

        public override void ProcessBeforeDelete(object item, AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (item is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)item;
                if (mail.To.Contains("TheBoss@work.com"))
                {
                    Outlook._Application outlookApp = ((AddinModule)this.Module).OutlookApp;

                    Outlook._JournalItem journalItem = null;

                    try
                    {
                        journalItem = (Outlook._JournalItem)outlookApp.CreateItem(Outlook.OlItemType.olJournalItem);
                        journalItem.Subject = "Oh oh! You deleted an email from the boss";
                        journalItem.Save();
                    }
                    finally
                    {
                        if (journalItem != null)
                            Marshal.ReleaseComObject(journalItem);
                    }
                }
            }
        }

        public override void ProcessUnload()
        {

        }

        public override void ProcessBeforeAutoSave(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is Outlook._MailItem)
            {
                Outlook._MailItem mail = (Outlook._MailItem)ItemObj;

                Outlook.UserProperties properties = null;
                Outlook.UserProperty newUserProperty = null;

                try
                {
                    properties = mail.UserProperties;
                    newUserProperty = properties.Add("AutoSaved", Outlook.OlUserPropertyType.olDateTime);
                    newUserProperty.Value = DateTime.Now;
                }
                finally
                {
                    if (newUserProperty != null)
                        Marshal.ReleaseComObject(newUserProperty);
                    if (properties != null)
                        Marshal.ReleaseComObject(properties);
                }
            }
        }

        public override void ProcessBeforeRead()
        {

        }

        public override void ProcessAfterWrite()
        {
            if (this.ItemObj is Outlook._TaskItem)
            {
                Outlook._TaskItem task = (Outlook._TaskItem)ItemObj;
                MessageBox.Show(task.MessageClass.ToString());
            }
        }
    }
}

